@@ -7,6 +7,7 @@ from imio.actionspanel import ActionsPanelMessageFactory as _
 from imio.actionspanel.interfaces import IContentDeletable
 from imio.actionspanel.utils import findViewableURL
 from imio.actionspanel.utils import unrestrictedRemoveGivenObject
+from imio.helpers.content import uuidsToObjects
 from imio.history.interfaces import IImioHistory
 from operator import itemgetter
 from plone import api
@@ -24,6 +25,7 @@ from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
 from zope.component import getAdapter
 from zope.component import getMultiAdapter
 from zope.component import getUtility
+from zope.dottedname.resolve import resolve
 from zope.i18n import translate
 
 import json
@@ -53,11 +55,12 @@ class ActionsPanelView(BrowserView):
             self.portal_url = self.portal.absolute_url()
             self.request.set('imio.actionspanel_portal_url_cachekey', self.portal_url)
             self.request.set('imio.actionspanel_portal_cachekey', self.portal)
-        self.SECTIONS_TO_RENDER = ('renderTransitions',
+        self.SECTIONS_TO_RENDER = ('renderFolderContents',
                                    'renderEdit',
                                    'renderExtEdit',
-                                   'renderOwnDelete',
+                                   'renderTransitions',
                                    'renderArrows',
+                                   'renderOwnDelete',
                                    'renderActions',
                                    'renderAddContent',
                                    'renderHistory')
@@ -81,7 +84,9 @@ class ActionsPanelView(BrowserView):
                  showHistory=False,
                  showHistoryLastEventHasComments=True,
                  showArrows=False,
+                 showFolderContents=False,
                  arrowsPortalTypeAware=False,
+                 markingInterface=None,
                  **kwargs):
         """
           Master method that will render the content.
@@ -104,11 +109,13 @@ class ActionsPanelView(BrowserView):
         self.showHistory = showHistory
         self.showHistoryLastEventHasComments = showHistoryLastEventHasComments
         self.showArrows = showArrows
+        self.showFolderContents = showFolderContents
         # arrowsPortalTypeAware will change the script used to
         # change object position.  It is used when several elements of
         # various portal_type are in the same container but we want to
         # order the elements of same portal_type together
         self.arrowsPortalTypeAware = arrowsPortalTypeAware
+        self.markingInterface = markingInterface
         self.kwargs = kwargs
         self.hasActions = False
         return self.index()
@@ -166,13 +173,23 @@ class ActionsPanelView(BrowserView):
             return ViewPageTemplateFile("actions_panel_transitions.pt")(self)
         return ''
 
+    def renderFolderContents(self):
+        """
+          Render a 'folder_contents' action.
+        """
+        if self.showFolderContents and \
+           (not self.markingInterface or self.isMarked(self.markingInterface, self.getCurrentFolder())) and \
+           self.mayFolderContents():
+            return ViewPageTemplateFile("actions_panel_folder_contents.pt")(self)
+        return ''
+
     def renderEdit(self):
         """
           Render a 'edit' action.  By default, only available when actions are displayed
           as icons because when it is not the case, we already have a 'edit' tab and that would
           be redundant.
         """
-        if self.showEdit and self.useIcons and self.mayEdit():
+        if self.showEdit and self.mayEdit():
             return ViewPageTemplateFile("actions_panel_edit.pt")(self)
         return ''
 
@@ -231,6 +248,15 @@ class ActionsPanelView(BrowserView):
         adapter = getAdapter(self.context, IImioHistory, 'workflow')
         return adapter.historyLastEventHasComments()
 
+    def mayFolderContents(self):
+        """
+          Method that check if folder_contents action has to be displayed.
+        """
+        if self.member.has_permission('List folder contents', self.context):
+            plone_view = getMultiAdapter((self.context, self.request), name='plone')
+            return bool(plone_view.displayContentsTab())
+        return False
+
     def mayEdit(self):
         """
           Method that check if special 'edit' action has to be displayed.
@@ -270,15 +296,20 @@ class ActionsPanelView(BrowserView):
         """ Sort the list of transitions by title """
         lst.sort(key=itemgetter('title'))
 
-    def getTransitions(self):
+    def getTransitions(self, caching=True):
         """
           This method is similar to portal_workflow.getTransitionsFor, but
           with some improvements:
           - we retrieve transitions that the user can't trigger, but for
             which he needs to know for what reason he can't trigger it;
           - for every transition, we know if we need to display a confirm
-            popup or not.
+            popup or not;
+          If caching=True, we will stored result in _transitions and use it
+          if method is called again.
         """
+        if caching:
+            if getattr(self, '_transitions', None):
+                return self._transitions
         res = []
         # Get the workflow definition for p_obj.
         workflow = self.request.get('imio.actionspanel_workflow_%s_cachekey' % self.context.portal_type, None)
@@ -341,6 +372,9 @@ class ActionsPanelView(BrowserView):
                     res.append(tInfo)
 
         self.sortTransitions(res)
+        if caching:
+            # store transitions in case getTransitions is called several times
+            setattr(self, '_transitions', res)
         return res
 
     def _transitionsToConfirmInfos(self):
@@ -446,6 +480,11 @@ class ActionsPanelView(BrowserView):
 
     def computeTriggerTransitionOnClick(self, transition):
         """ """
+        transition_ids = [tr['id'] for tr in self.getTransitions()]
+        # if transition is no more available, this means that element's transition
+        # was already triggered by another user or in another tab, we just refresh the page
+        if not transition or transition['id'] not in transition_ids:
+            return 'window.location.href=window.location.href;'
         if not transition['confirm']:
             return "triggerTransition(baseUrl='{0}', viewName='@@triggertransition', transition='{1}', this);".format(
                 self.context.absolute_url(),
@@ -471,49 +510,35 @@ class ActionsPanelView(BrowserView):
 
     def listObjectButtonsActions(self):
         """
-          Return a list of object_buttons actions coming from portal_actions.
+          Return a list of object_buttons actions coming from portal_actions/portal_types.
         """
         actionsTool = api.portal.get_tool('portal_actions')
         typesTool = api.portal.get_tool('portal_types')
-        # we only want object_buttons, so ignore other categories and providers
-        IGNORABLE_CATEGORIES = ['site_actions', 'object', 'controlpanel/controlpanel_addons', 'workflow',
-                                'portal_tabs', 'global', 'document_actions', 'user', 'folder_buttons', 'folder',
-                                'jqueryui_panels']
-        IGNORABLE_PROVIDERS = ['portal_workflow', ]  # 'portal_types' ?
-
-        if self.ACCEPTABLE_ACTIONS:  # we know the set of actions. Only work with them. Faster !
-            actions = [act for act in (actionsTool.listActions(categories='object_buttons') +
-                                       tuple(typesTool.listActions(object=self.context, category='object_buttons')))
-                       if act.id in self.ACCEPTABLE_ACTIONS]
-            ec = actionsTool._getExprContext(self.context)
-            actions = [ActionInfo(action, ec) for action in actions]
-            objectButtonActions = [act for act in actions if act['visible'] and act['allowed'] and act['available']]
-        else:
-            allActions = actionsTool.listFilteredActionsFor(self.context,
-                                                            ignore_providers=IGNORABLE_PROVIDERS,
-                                                            ignore_categories=IGNORABLE_CATEGORIES)
-            objectButtonActions = []
-            if 'object_buttons' in allActions:
-                objectButtonActions = allActions['object_buttons']
+        # filter acceptable/ignorable actions before evaluating the TAL expression
+        actions = [act for act in (actionsTool.listActions(categories=['object_buttons']) +
+                                   tuple(typesTool.listActions(object=self.context, category='object_buttons')))
+                   if (self.ACCEPTABLE_ACTIONS and act.id in self.ACCEPTABLE_ACTIONS) or
+                      (not self.ACCEPTABLE_ACTIONS and act.id not in self.IGNORABLE_ACTIONS)]
+        ec = actionsTool._getExprContext(self.context)
+        actions = [ActionInfo(action, ec) for action in actions]
+        objectButtonActions = [act for act in actions if act['visible'] and act['allowed'] and act['available']]
 
         res = []
         for action in objectButtonActions:
-            if (self.ACCEPTABLE_ACTIONS and action['id'] in self.ACCEPTABLE_ACTIONS) or \
-               (not self.ACCEPTABLE_ACTIONS and not action['id'] in self.IGNORABLE_ACTIONS):
-                act = action.copy()
-                # We try to append the url of the icon of the action
-                # look on the action itself
-                if act['icon']:
-                    # make sure we only have the action icon name not a complete
-                    # path including portal_url or so, just take care that we do not have
-                    # an image in a static resource folder
-                    splittedIconPath = act['icon'].split('/')
-                    if len(splittedIconPath) > 1 and '++resource++' in splittedIconPath[-2]:
-                        # keep last 2 parts of the path
-                        act['icon'] = '/'.join((splittedIconPath[-2], splittedIconPath[-1], ))
-                    else:
-                        act['icon'] = splittedIconPath[-1]
-                res.append(act)
+            act = action.copy()
+            # We try to append the url of the icon of the action
+            # look on the action itself
+            if act['icon']:
+                # make sure we only have the action icon name not a complete
+                # path including portal_url or so, just take care that we do not have
+                # an image in a static resource folder
+                splittedIconPath = act['icon'].split('/')
+                if len(splittedIconPath) > 1 and '++resource++' in splittedIconPath[-2]:
+                    # keep last 2 parts of the path
+                    act['icon'] = '/'.join((splittedIconPath[-2], splittedIconPath[-1], ))
+                else:
+                    act['icon'] = splittedIconPath[-1]
+            res.append(act)
         return res
 
     def triggerTransition(self, transition, comment, redirect=True):
@@ -563,6 +588,21 @@ class ActionsPanelView(BrowserView):
         """
         return findViewableURL(self.context, self.request, self.member)
 
+    def getCurrentFolder(self):
+        plone_view = getMultiAdapter((self.context, self.request), name='plone')
+        return plone_view.getCurrentFolder()
+
+    def isMarked(self, interface_name, context=None):
+        if interface_name is None:
+            return True
+        if context is None:
+            context = self.context
+        try:
+            interface_class = resolve(interface_name)
+        except Exception:
+            return False
+        return interface_class.providedBy(context)
+
 
 class DeleteGivenUidView(BrowserView):
     """
@@ -587,16 +627,12 @@ class DeleteGivenUidView(BrowserView):
             redirect = False
         elif redirect == '1':
             redirect = True
-        # Get the object to delete
-        # try to get it from the portal_catalog
-        catalog_brains = self.context.portal_catalog(UID=object_uid)
-        # if not found, try to get it from the uid_catalog
-        if not catalog_brains:
-            catalog_brains = self.context.uid_catalog(UID=object_uid)
-        # if nto found at all, raise
-        if not catalog_brains:
+        # Get the object to delete, if not found using UID index,
+        # try with contained_uids index
+        objs = uuidsToObjects(uuids=[object_uid], check_contained_uids=True)
+        if not objs:
             raise KeyError('The given uid could not be found!')
-        obj = catalog_brains[0].getObject()
+        obj = objs[0]
 
         # we use an adapter to manage if we may delete the object
         # that checks if the user has the 'Delete objects' permission
@@ -654,5 +690,9 @@ class AsyncActionsPanelView(BrowserView):
     def __call__(self, **kwargs):
         """ """
         kwargs.update(self._convert_form_values())
+        # remove '_' from kwargs, it is the ajax_load id
+        # if we leave it, ram.cached __call__ is never cached as this value is always different
+        if '_' in kwargs:
+            kwargs.pop('_')
         rendered_actions_panel = self.context.restrictedTraverse('@@actions_panel')(**kwargs)
         return rendered_actions_panel
